import pytest
import time
import sys
import os
from unittest.mock import patch, MagicMock

# Ensure the shop module is in the Python path
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from shop.main import retrieve_modules_full_list, load_from_cache, save_to_cache, fetch_repos_from_github, fetch_repos_from_gitlab


@patch("shop.main.load_from_cache")
@patch("shop.main.save_to_cache")
def test_retrive_real_data(mock_save_cache, mock_load_cache):
    # Mock cache data
    mock_load_cache.return_value = (None, 0)

    repos = retrieve_modules_full_list(force_refresh=True, initial_page=1, per_page=30)
    assert len(repos) > 0   
    

@patch("shop.main.load_from_cache")
@patch("shop.main.save_to_cache")
@patch("shop.main.fetch_repos_from_github")
@patch("shop.main.fetch_repos_from_gitlab")
@patch("time.time", return_value=1609459200)  # Mock time to a fixed timestamp
def test_retrieve_modules_full_list(mock_time, mock_fetch_gitlab, mock_fetch_github, mock_save_cache, mock_load_cache):
    # Mock cache data
    mock_load_cache.return_value = (None, 0)
    
    # Mock GitHub response
    mock_fetch_github.return_value = [
        {
            "name": "tuneit-mod-example",
            "owner": {"login": "example"},
            "default_branch": "main",
            "stargazers_count": 10,
            "description": "Example description",
            "html_url": "https://github.com/example/tuneit-mod-example",
            "full_name": "example/tuneit-mod-example"
        }
    ]
    
    # Mock GitLab response
    mock_fetch_gitlab.return_value = [
        {
            "name": "tuneit-mod-example",
            "path_with_namespace": "example/tuneit-mod-example",
            "default_branch": "main",
            "star_count": 5,
            "description": "Example description",
            "web_url": "https://gitlab.com/example/tuneit-mod-example"
        }
    ]
    
    # Mock parse_module_yaml to return a dummy module info
    with patch("shop.main.parse_module_yaml", return_value=[{"name": "Example Module", "description": "Example module description"}]):
        repos = retrieve_modules_full_list(force_refresh=True, initial_page=1, per_page=30)
    
    assert len(repos) == 2
    assert repos[0]["source"] == "github"
    assert repos[0]["repo_name"] == "tuneit-mod-example"
    assert repos[0]["stars"] == 10
    assert repos[0]["module_yaml"]["name"] == "Example Module"
    
    assert repos[1]["source"] == "gitlab"
    assert repos[1]["repo_name"] == "tuneit-mod-example"
    assert repos[1]["stars"] == 5
    assert repos[1]["module_yaml"]["name"] == "Example Module"
    
    mock_save_cache.assert_called_once()

@patch("shop.main.load_from_cache")
@patch("time.time", return_value=1609459200)  # Mock time to a fixed timestamp
def test_retrieve_modules_full_list_cache(mock_time, mock_load_cache):
    # Mock cache data
    mock_load_cache.return_value = ([{"repo_name": "cached-repo"}], 1609455600)
    
    repos = retrieve_modules_full_list(force_refresh=False, initial_page=1, per_page=30)
    
    assert len(repos) == 1
    assert repos[0]["repo_name"] == "cached-repo"
